#!/usr/bin/perl  -w
#$Header: /usr/cvs/lui/lui/lib/luiapi.pm,v 1.13 2001/06/19 20:02:08 mchasal Exp $

# Copyright (c) 2000, International Business Machines
#
# This software may be used and distributed according to the terms
# of the GNU Public License, incorporated herein by reference.
#
# The entire text of the GNU Public License is included in this
# package in the COPYING file; or can be found online at:
# http://www.Linux.org/info.gnu.html
#
# LUI API
#
# Authors: Richard Ferri,
#          Jeremy Hughes
#          Laura Schornack
#          Norm Nott
#
#####################################################################
#                                                                   #
# Module:  luiapi                                                   #
#                                                                   #
# Description:                                                      #
#                                                                   #
# Subroutines:                                                      #
#  lui_read_machine_object  lui_make_machine  lui_make_resource     #
#  lui_allocate_resource   lui_delete_machine  lui_delete_resource  #
#  lui_list_machine  lui_list_resource                              #
#  lui_unallocate_resource  lui_change_install_mode                 # 
#                                                                   #
# Notes:                                                            #
#                                                                   #
#####################################################################

package luiapi;

use FindBin qw($Bin);
use lib "$Bin";
use Socket;
use vars qw($BASE @ISA @EXPORT @EXPORT_OK);
use Exporter;
@ISA = qw(Exporter);
@EXPORT=qw(%machineObject @members %Groups);

# Here we set the base for all things

$BASE = "$Bin/..";

require "lim.pl";

#--------------------------------------------------------------------#
# Global definitions                                                 #
#--------------------------------------------------------------------#

$LIMDIR="/tftpboot/lim";
$CLIENT="client";
$SERVER="server";
$NOK=1;
$OK=0;

#--------------------------------------------------------------------#
# Subroutines                                                        #
#---------------------------------------#----------------------------#

#####################################################################
#                                                                   #
# Subroutine:   lui_read_machine_object                             #
#                                                                   #
# Description:                                                      #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_read_machine_object{
  my ($name,$type,$dir)=@_;
  my $localline;

  if ($dir eq "") {
     $dir = $LIMDIR;
  }

  $filename = $dir . "/" . $name . "." . $type;
  if (!open (MACHINE_OBJECT, "<$filename")){
        print STDERR "Could not open the machine object file $filename exiting\n";
        return 1;
  } else {
        $localline=<MACHINE_OBJECT>;
        close MACHINE_OBJECT;
  }

  chop($localline);
  if ($type eq $SERVER){
     ($Sip,$Snetmask,$Spbsstring,$Snumproc,$Sreboot)=split(/:/,$localline);
     $machineObject{$name}{$type}{"IP"} = $Sip;
     $machineObject{$name}{$type}{"NETMASK"} = $Snetmask;
     $machineObject{$name}{$type}{"PBS_STRING"} = $Spbsstring;
     $machineObject{$name}{$type}{"NUM_PROC"} = $Snumproc;
     $machineObject{$name}{$type}{"REBOOT"} = $Sreboot;
  }
  else {
    my ($Nip, $Nnetmask, $Nha, $Ntype, $Nadapter, $Ndefault, $Nhostname, $Ngateway, $Nnumproc, $NPBSstr, $MACid, $Reboot) = split (/:/,$localline);

    $machineObject{$name}{$type}{"IP"} = $Nip;
    $machineObject{$name}{$type}{"NETMASK"} = $Nnetmask;
    $machineObject{$name}{$type}{"HA"} = $Nha;
    $machineObject{$name}{$type}{"TYPE"} = $Ntype;
    $machineObject{$name}{$type}{"ADAPTER"} = $Nadapter;
    $machineObject{$name}{$type}{"DEFROUTE"} = $Ndefault;
    $machineObject{$name}{$type}{"HOSTNAME"} = $Nhostname;
    $machineObject{$name}{$type}{"GATEWAY"} = $Ngateway;
    $machineObject{$name}{$type}{"NUM_PROC"} = $Nnumproc;
    $machineObject{$name}{$type}{"PBS_STRING"} = $NPBSstr;
    $machineObject{$name}{$type}{"MAC_ID"} = $MACid;
    $machineObject{$name}{$type}{"REBOOT"} = $Reboot;
  }
  return 0;
} # End lui_read_machine_object

#####################################################################
#                                                                   #
# Subroutine:   lui_make_machine                                    #
#                                                                   #
# Description:  Define a client or server machine.                  #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
# Notes:  The attrs hash should contain the attributes of the       #
#         object.  The flags hash contains everything else.         #
#                                                                   #
#####################################################################
sub lui_make_machine {
    local (*flags, *attrs) = @_;
    local $mtype,%gflags;


    # see if this machine def already exists
    if ( ($mtype=&machine_exists($attrs{'name'})) ) {
        print "error -- The machine '$attrs{'name'}' already exists, exiting without creating a machine resource\n";
        return $NOK;
    }

    # for now the install adapter must be eth0
    if ( ($attrs{'inst_adptr'} ne "") && ($attrs{'inst_adptr'} ne "eth0") ){
        print "error -- The \'eth0\' installation adapter is currently the only supported adapter.\n";
        return $NOK;
    }

    if ($attrs{'type'} eq $SERVER){
        if (&check_server_services){	# insure server services are running 
            return $NOK;
        }
	if ($attrs{reboot} eq "" ) { 	# Set the default reboot value if not given
		$attrs{reboot}="false";
	}
        if (&write_server_obj){        # write the server object
            return $NOK;
        }
        print "\nsuccessfully created the machine object named $attrs{'name'}\n";

        if ($flags{G}) {
            $gflags{m}=$attrs{'name'};
            $gflags{g}=$flags{G};
            $gflags{t}="machine";
            if (&lui_make_group(\%gflags)) {
                return $NOK;
            }
        }
    }

    if ($attrs{'type'} eq $CLIENT){
	
        if (&get_macaddr){    # get the mac address from /etc/MAC.info 
            return $NOK;
        }
        if (&add_bootp){    # create the bootp entry for the client
            return $NOK;
        }
        if (&create_client_common){  # if this is first client, create
            return $NOK;    # common root FS portion
        }
        if (&create_client_root){   # create the remote root fs for the client
            return $NOK;
        }
        if (&create_client_fstab){   # create the remote etc/fstab for booting
            return $NOK;
        }
        if (&create_client_name){   # create the remote client name file
            return $NOK;
        }
        $arch = `$ARCH`;
        chop($arch);
        if ($arch ne "ppc") {   # Don't need pxe for Power PC
            if (&create_client_pxelinux){   # create remote client pxelinux file
                return $NOK;
            }
        }

	if ($attrs{reboot} eq "" ) { 	# Set the default reboot value if not given
		$attrs{reboot}="default";
	}

	# set default number of processors if none specified.
    	if($attrs{num_proc}=="") {
        	$attrs{num_proc} = 1;
    	}            
        $line="$attrs{'ip_address'}:$attrs{'netmask'}:$attrs{'mac_address'}:$INSTALL_MODE:$attrs{'inst_adptr'}:$attrs{'def_route'}:$attrs{'lhostname'}:$attrs{'inst_gateway'}:$attrs{'num_proc'}:$attrs{'pbs_string'}:$attrs{'mac_id'}:$attrs{reboot}:";

        if (&write_client_obj("$LIMDIR/$attrs{'name'}.client",$line)){	
            return $NOK;
        }
        print "Successfully created the machine object $attrs{'name'}.\n";

        if ($flags{G}) {
            $gflags{m}=$attrs{'name'};
            $gflags{g}=$flags{G};
            $gflags{t}="machine";
            if (&lui_make_group(\%gflags)) {
                return $NOK;
            }
        }
    }
    return $OK;
} # End lui_make_machine

######### start lui_make_machine subroutines #######################

#####################################################################
#                                                                   #
# Subroutine:   create_client_common                                #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################
sub create_client_common {

    if (-d "$COMMON"){        # if there is a common root fs, exit 
        return $OK;
    }

    `$UMASK 022`;
    `$MKDIR $COMMON 2>/dev/null`;
    if ($? != 0){
        print "Error -- could not create common client remote root directory in $COMMON, exiting.\n";
        return $NOK;
    }

    foreach $dir (@COMMONDIRS){
        `$MKDIR $COMMON/$dir 2> /dev/null`;
        if ($? != 0){
            print "Error -- could not create $COMMON/$dir, exiting.\n";
            return $NOK;
	}
        `$TOUCH $COMMON/$dir/junk`;
    }

    print "About to create common root file system, this might take a minute or two.\n";
    foreach $dir(@COMMONDIRS){
        `$CP -a  /$dir $COMMON`;
         if ($? != 0){
             print "Error -- unable to populate common client remote root file system, exiting.\n";
             return $NOK;
         }
    }

    print "\nSuccessfully created the common root file system, continuing.\n";
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   create_client_root                                  #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################

sub create_client_root{
    #
    # About to create the remote root file system in /tftpboot for the client
    # I stole most of the code in this section from the etherboot project,
    # currently owned by Ken Yap.  Thanks, Ken
    #

    `$UMASK 022`;
    `$MKDIR /tftpboot/$attrs{'ip_address'} 2>/dev/null`;
    if ($? != 0){
        print "Error -- could not create client remote root directory in /tftpboot/$attrs{'ip_address'}, exiting.\n";
        return $NOK;
    }

    foreach $dir (@ROOTDIRS){
        `$MKDIR /tftpboot/$attrs{'ip_address'}/$dir 2> /dev/null`;
        if ($? != 0){
            print "Error -- could not create /tftpboot/$attrs{'ip_address'}/$dir, exiting.\n";
            return $NOK;
        }
    }

    @vars=`cd /var;$FIND . -type d -print`;
    foreach $var(@vars){
        chop $var;
        `$MKDIR -p /tftpboot/$attrs{'ip_address'}/var/$var`;
    }

    `$CHMOD 1777 /tftpboot/$attrs{'ip_address'}/tmp`;

    print "About to create root file system for client $attrs{'name'}, this might take a minute or two.\n";

    `$CP -a  /etc /tftpboot/$attrs{'ip_address'}`;
    unlink </tftpboot/$attrs{'ip_address'}/etc/sysconfig/network-scripts/ifcfg-eth*>;
    `$ECHO $attrs{'lhostname'} > /tftpboot/$attrs{'ip_address'}/etc/HOSTNAME`;
    if ($? != 0){
        print "Error -- unable to populate remote root file system for client, exiting.\n";
        return $NOK;
    }
  
    for $common_dir(@COMMONDIRS){
        `$CP -dlr $COMMON/$common_dir/ /tftpboot/$attrs{'ip_address'}/`;
        if ($? != 0){
            print "Error -- unable to link $common_dir to common directory for client, exiting.\n";
            return $NOK;
        }
    }

# SuSE network config is done using /etc/rc.config whereas RedHat network
# config is accomplished by configuration files in /etc/sysconfig.

    if ($DISTRIB_CLIENT eq "SuSE") {
         `$ECHO "#! /bin/sh" >> /tftpboot/$attrs{'ip_address'}/etc/init.d/luilocal`;
         `$ECHO "$Bin/clone" >> /tftpboot/$attrs{'ip_address'}/etc/init.d/luilocal`;
         `$CHMOD 544 /tftpboot/$attrs{'ip_address'}/etc/init.d/luilocal`;
         if ($? != 0){
            print "Error -- unable to modify etc/rc.d/local in remote root filesystem, exiting.\n";
            return $NOK;
         }
         `$LN -s ../luilocal /tftpboot/$attrs{'ip_address'}/etc/init.d/rc2.d/S98local`;
         `$LN -s ../luilocal /tftpboot/$attrs{'ip_address'}/etc/init.d/rc3.d/S98local`;
         if (&create_suseconfig) {
             return $NOK;
         }
      } else { # RedHat and MSC ditributions
        `$CP $BASE/templates/network /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network`;
        if ($? != 0){
            print "error -- unable to copy etc/sysconfig/network from $BASE/templates to remoteroot filesystem, exiting\n";
            return $NOK;
        }

        `$ECHO "HOSTNAME=$attrs{'lhostname'}" >> /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network`;
        `$ECHO "GATEWAYDEV=eth0" >> /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network`;
        `$ECHO "GATEWAY=$attrs{'def_route'}" >> /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network`;

        if ($? != 0){
            print "Error -- unable to modify etc/sysconfig/network in remote root filesystem, exiting.\n";
            return $NOK;
        }

        `$CP $BASE/templates/ifcfg-eth0  /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network-scripts/ifcfg-eth0`;
        if ($? != 0){
            print "Error -- unable to copy etc/sysconfig/network-scripts/ifcfg-eth0 from $BASE/templates to remote root filesystem, exiting.\n";
            return $NOK;
        }

        `$ECHO "NETMASK=$attrs{'netmask'}" >> /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network-scripts/ifcfg-eth0`;
        `$ECHO "IPADDR=$attrs{'ip_address'}" >> /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network-scripts/ifcfg-eth0`;

        ($network,$broadcast)=&network_ip($attrs{'ip_address'}, $attrs{'netmask'});

        `$ECHO "NETWORK=$network" >> /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network-scripts/ifcfg-eth0`;
        `$ECHO "BROADCAST=$broadcast" >> /tftpboot/$attrs{'ip_address'}/etc/sysconfig/network-scripts/ifcfg-eth0`;
        if ($? != 0){
            print "Error -- unable to modify etc/sysconfig/network-scripts/ifcfg-eth0 in remote root filesystem, exiting.\n";
            return 1;
        }

        `$ECHO "$BASE/bin/clone" >> /tftpboot/$attrs{'ip_address'}/etc/rc.d/rc3.d/S99local`;
        if ($? != 0){
            print "Error -- unable to modify etc/rc.d/rc3.d/S99local in remote root filesystem, exiting.\n";
            return $NOK;
        }
        if (! ` $GREP "^$BASE/bin/clone" /tftpboot/$attrs{'ip_address'}/etc/rc.d/rc5.d/S99local` ) {
        `$ECHO "$BASE/bin/clone" >> /tftpboot/$attrs{'ip_address'}/etc/rc.d/rc5.d/S99local`;
            if ($? != 0){
                print "Error -- unable to modify etc/rc.d/rc5.d/S99local in remote root filesystem, exiting.\n";
                return $NOK;
            }
        }
        if ($DISTRIB_CLIENT eq "MSC") {
            `chmod 777 /tftpboot/$attrs{'ip_address'}/etc/rc.d/rc3.d/S99local`;
            `chmod 777 /tftpboot/$attrs{'ip_address'}/etc/rc.d/rc5.d/S99local`;
        }
    } # End of non-SuSE network config

    if (! defined $KEEP_SERVICES{"ALL"}) {
      if ($DISTRIB_CLIENT eq "SuSE") {
         $servicesdir = "/tftpboot/$attrs{'ip_address'}/etc/init.d";
      } else {
         $servicesdir = "/tftpboot/$attrs{'ip_address'}/etc/rc.d/init.d";
      }
      foreach $service (`ls $servicesdir`){
         chop $service;
         if (! defined $KEEP_SERVICES{$service}) {
           `$RM $servicesdir/$service > /dev/null 2>&1`;
         }
      }
    }
    

    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   check_server_services                               #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################
sub check_server_services{

    # check for tftp
    my $tftp_notrunning=1;
    my $disabled=0;
    my $line;
    if(`$PS aux | $GREP tftp | $GREP -v grep 2> /dev/null`){	
	$tftp_notrunning=0;
    } elsif (`$PS aux | $GREP inetd | $GREP -v grep 2> /dev/null`
             && `$GREP tftp /etc/inetd.conf 2> /dev/null | $GREP -v \\#` ne ""){
	$tftp_notrunning=0;
    } elsif (`$PS aux | $GREP xinetd | $GREP -v grep 2> /dev/null`
	     && -e "/etc/xinetd.d/tftp"){
			open(TFTP_FILE,"</etc/xinetd.d/tftp");
			while($line = <TFTP_FILE>){
				if ($line =~ /disable\s*=\s*yes/){
					$disabled=1;
				}
			}
			if (!$disabled) {
				$tftp_notrunning=0;
			}
    }
    if ($tftp_notrunning) {
            print "\nWarning -- it appears that tftp is not running on this system.\n";
            print "I will continue trying to create the server object, but be aware that tftp must be running for node installation.\n";
    } else {
            print "\ntftp appears to be in working order, which is good.\n";
    }

    # check for nfsd
    if(`$PS aux | $GREP nfsd | $GREP -v grep 2> /dev/null` eq ""){	
        print "\nWarning -- it appears that nfsd is not running on this system.\n";
        print "I will continue trying to create the server object, but be aware that nfsd must be running for node installation.\n";
    } else {
        print "\nnfsd appears to be in working order on this system, which is good.\n";
    }

    # check for tftpboot dir
    if(! `$GREP -w "\/tftpboot" /etc/exports`){
        print "\nWarning -- it appears that /tftpboot is not exported on this system.\n";
        print "I will continue trying to create the server object, but be aware that /tftpboot must be exported for node installation.\n";
    }

    # check for /usr
    if(! `$GREP -w "\/usr" /etc/exports`){
        print "\nWarning -- it appears that /usr is not exported on this system.\n";
        print "I will continue trying to create the server object, but be aware that /usr must be exported for node installation.\n";
    }
    # 
    # creating /tftpboot is not really a service, but it belongs here
    #

    if(-d "/tftpboot"){
        print "\n/tftpboot directory already exists, which is good.\n";
    } else {
        `$UMASK 022`;
        `$MKDIR /tftpboot`;
        if ($? != 0){
            print "Error -- could not create /tftpboot directory, exiting.\n";
            return $NOK;
        } else {
            print "Successfully created /tftpboot directory.\n";
        }
    }

    if (-d $LIMTAR){
    } else {
        `$MKDIR $LIMTAR`;
    }

    if (-d $PXELINUX){
    } else{
        `$MKDIR $PXELINUX`;
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:    write_server_obj                                   #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################
sub write_server_obj{

    `$MKDIR -p $LIMLOG`;
    `$ECHO "$attrs{'ip_address'}:$attrs{'netmask'}:$attrs{'num_proc'}:$attrs{'pbs_string'}:$attrs{'reboot'}:" > $LIMDIR/$attrs{'name'}.$attrs{'type'}`;
    if ($? != 0){
        return $NOK;
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:    create_client_fstab                                #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################
sub create_client_fstab{

    chop($server_tmp=`cd $LIMDIR;$LS *.server`);
    ($server_fn,$rest)=split(/\./,$server_tmp);
 
    luiapi::lui_read_machine_object($server_fn,$SERVER);
    &assign_server_values($server_fn);

    `$ECHO "$server_ipaddr:/tftpboot/$attrs{'ip_address'} / nfs exec,dev,suid,rw 1 1 " > /tftpboot/$attrs{'ip_address'}/etc/fstab`;
    `$ECHO "$server_ipaddr:/usr /usr nfs exec,dev,suid,rw 1 1 " >> /tftpboot/$attrs{'ip_address'}/etc/fstab`;
    `$ECHO "$server_ipaddr:/tftpboot/lim /lim nfs exec,dev,suid,rw 1 1 " >> /tftpboot/$attrs{'ip_address'}/etc/fstab`;
    `$ECHO "$server_ipaddr:$LIMTAR /tar nfs exec,dev,suid,rw 1 1 " >> /tftpboot/$attrs{'ip_address'}/etc/fstab`;
    `$ECHO "$server_ipaddr:$LIMRPM /rpm nfs exec,dev,suid,rw 1 1 " >> /tftpboot/$attrs{'ip_address'}/etc/fstab`;
    `$CAT $BASE/templates/fstab >> /tftpboot/$attrs{'ip_address'}/etc/fstab`;

    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:    create_client_name                                 #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################}
sub create_client_name{

    `$ECHO $attrs{'name'} > /tftpboot/$attrs{'ip_address'}/namefile`;
    if ($? != 0){
        print "Error -- could not create client name file in /tftpboot/$attrs{'ip_address'}/namefile, exiting.\n";
        return $NOK;
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   create_client_pxelinux                              #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################
sub create_client_pxelinux{

    ($n1,$n2,$n3,$n4)=split(/\./,$attrs{'ip_address'});
    $x1=sprintf("%lx",$n1);
    $x2=sprintf("%lx",$n2);
    $x3=sprintf("%lx",$n3);
    $x4=sprintf("%lx",$n4);
    $x1 =~ tr/a-f/A-F/;
    $x2 =~ tr/a-f/A-F/;
    $x3 =~ tr/a-f/A-F/;
    $x4 =~ tr/a-f/A-F/;

    if ($n1 <= 15){
        $x1 = "0" . $x1;
    }

    if ($n2 <= 15){
        $x2 = "0" . $x2;
    }

    if ($n3 <= 15){
        $x3 = "0" . $x3;
    }

    if ($n4 <= 15){
        $x4 = "0" . $x4;
    }

    $pxeclient_name= $x1 . $x2 . $x3 . $x4; 
    chop($server_tmp=`cd $LIMDIR;$LS *.server`);
    ($server_fn,$rest)=split(/\./,$server_tmp);

    luiapi::lui_read_machine_object($server_fn,$SERVER);
    &assign_server_values($server_fn);

    `$ECHO "DEFAULT bzImage" > $PXELINUX/$pxeclient_name`;
    `$ECHO "APPEND vga=extended root=/tftpboot/$attrs{'ip_address'}" >> $PXELINUX/$pxeclient_name`;
    `$ECHO "PROMPT 1" >> $PXELINUX/$pxeclient_name`;
    `$ECHO "TIMEOUT 50" >> $PXELINUX/$pxeclient_name`;

    return $OK; 
}

#####################################################################
#                                                                   #
# Subroutine:  add_bootp                                            #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################
sub add_bootp{
	
    if (&add_bootp_entry($attrs{'ip_address'},$attrs{'name'},$attrs{'inst_gateway'},$attrs{'lhostname'},$attrs{'netmask'},$attrs{'mac_address'})){
        return $NOK;
    } else{
        return $OK;
    }
}
#####################################################################
#                                                                   #
# Subroutine:  get_macaddr                                          #
#                                                                   #
# Description: Looks up the mac id given and finds the mac address  #
#                                                                   #
#####################################################################
sub get_macaddr{
	# If a mac_id was specified, get the address from the file
	if (defined $attrs{'mac_id'} && defined $attrs{'mac_address'}) {
		printf "You cannot specify both the MAC address and the MAC id.\n";
		return $NOK;
	}
	if (defined $attrs{'mac_id'}) {
		# Grep for the id followed by whitespace
		my $line=`$GREP -e "^$attrs{'mac_id'}\[ 	\]" $MACINFO 2>/dev/null`;
		chop $line;
		my ($id,$mac)=split(/\s+/,$line);
		# Get rid of colons
		$mac=~s/://g;
		# Check that the address is 12 hex digits
		if ($mac =~ /^[a-fA-F0-9]{12}$/) {
			# If so put it in the mac address field
			$attrs{'mac_address'}=$mac;
			return $OK;
		} else {
			# If not, fail
			printf "MAC address $mac for id $attrs{'mac_id'} in $MACINFO is missing or incorrect.\n";
			return $NOK;
		}
	}
	# Get rid of colons, if not from MAC.info
	$attrs{'mac_address'}=~s/://g;
	return $OK;
}

#####################################################################
#                                                                   #
# Subroutine: network_ip                                            #
#                                                                   #
# Description:                                                      #
#                                                                   #
#####################################################################
sub network_ip ($ipaddr,$netmask){
    ($ipaddr, $netmask) = @_;

    ($n1,$n2,$n3,$n4)=split(/\./, $ipaddr);
    ($m1,$m2,$m3,$m4)=split(/\./, $netmask);
    $nn1= int($n1) & int($m1);
    $nn2= int($n2) & int($m2);
    $nn3= int($n3) & int($m3);
    $nn4= int($n4) & int($m4);
    $network_addr = "$nn1.$nn2.$nn3.$nn4";
    $bcast = 256 - int ($m4) -1;
    $bcast = $bcast + $nn4;
    $bcast_addr= "$nn1.$nn2.$nn3.$bcast";
    @net[0]=$network_addr; 
    @net[1]=$bcast_addr;

    return @net;
}

#############  End lui_make_machine subroutines ##########################

#####################################################################
#                                                                   #
# Subroutine:   lui_make_resource                                   #
#                                                                   #
# Description:  Define a client installation resource.              #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
# Notes:  The attrs hash should contain the attributes of the       #
#         object.  The flags hash contains everything else.         #
#         The '-c' option is contained in the flags hash.           #
#                                                                   #
#                                                                   #
#####################################################################
sub lui_make_resource {
    local (*flags, *attrs) = @_;
    local $rtype,%gflags;

    #    Check type flag for validity

    if (! grep(/$attrs{type}/,values %RESOURCE_TYPE) ){
            print "Error -- the resource type \'$attrs{'type'}\' is not valid.\n";
            return $NOK;
    }

    # Check if already exists
    if ( ($rtype=&resource_exists($attrs{'name'})) ) {
        print "Error -- the resource '$attrs{'name'}' already exists, exiting without creating a(n) $attrs{'type'} resource.\n";
        return $NOK;
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{file}"){
        if (&create_tar_file){
            return $NOK;
        }
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{kernel}"){
        if (&create_kernel){
            return $NOK;
        }
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{map}"){
        if (&create_map){
            return $NOK;
        }
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{exit}"){
        if (&create_exit){
            return $NOK;
        }
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{postinstall}"){
        if (&create_postinstall){
            return $NOK;
        }
    }


    if ($attrs{'type'} eq "$RESOURCE_TYPE{rpm}"){
        if (&create_rpm){
            return $NOK;
        }
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{disk}"){
        if (&create_disk){
             return $NOK;
        }
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{source}"){
        if (&create_source){
            return $NOK;
        }
    }

    if ($attrs{'type'} eq "$RESOURCE_TYPE{ramdisk}"){
        if (&create_ramdisk){
            return $NOK;
        }
    }

    if (&write_resource_obj){
        return $NOK;
    }

    if ($flags{G}) {
        $gflags{m}=$attrs{'name'};
        $gflags{g}=$flags{G};
        $gflags{t}="resource";
        if (&lui_make_group(\%gflags)) {
            return $NOK;
        }
    }

   return $OK;

} # End lui_make_resource

#############  Start lui_make_resource subroutines #######################

#####################################################################
#                                                                   #
# Subroutine:  create_tar_file                                      #
#                                                                   #
# Description:  Create a tar file.                                  #
#                                                                   #
#####################################################################
sub create_tar_file{

    # if the create_tar value is set then DO NOT create a tar file
    #   ?????
    if ($flags{c}){
        return $OK;
    }

    print "About to tar the $attrs{'source_path_name'} file system, this might take a minute.\n";
    
    #
    # if not the root file system, tar everything in the fs to $LIMTAR
    #
    if ($attrs{'source_path_name'} ne "/"){
        `cd $attrs{'source_path_name'}; $TAR -zlcf $LIMTAR/$attrs{'name'}.tgz .`;
        if ($? != 0){
            print "Error -- tarring the $attrs{'source_path_name'} file system failed, exiting.\n";
            return $NOK;
        }
    } else {
#
# if the root file system, tar selected directories to $LIMTAR
#
        if ($flags{f} ne ""){
		`cd /; $TAR -zlcf $LIMTAR/$attrs{'name'}.tgz / --exclude=tftpboot --exclude=tmp --exclude=proc`;
	}	
	else{
		if ($DISTRIB_CLIENT eq "RedHat" || $DISTRIB_CLIENT eq "SuSE") {
			`cd /; $TAR -zlcf $LIMTAR/$attrs{'name'}.tgz bin dev sbin etc lib mnt `;		 
		} elsif ($DISTRIB_CLIENT eq "MSC") {
			`cd /; $TAR -zlcf $LIMTAR/$attrs{'name'}.tgz bin dev sbin etc lib mnt webmin-0.80 `;		 
	 	}
	}
    }

    if ($? != 0){
        print "Error -- tarring the $attrs{'source_path_name'} file system failed, exiting.\n";
        return $NOK;	
    } else{
        print "Successfully created the $attrs{'source_path_name'} tar file, continuing.\n";
    }

    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:  create_kernel                                        #
#                                                                   #
# Description:  Create kernel resource.                             #
#                                                                   #
#####################################################################
sub create_kernel{

    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.kernel`;
    if ($? != 0){
        print "Error -- copying the kernel resource to $LIMTAR failed, exiting.\n";
        return $NOK;
    } else {
        print "Successfully copied the kernel resource $attrs{'name'}, continuing.\n";
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   create_source                                       #
#                                                                   #
# Description:  Copy source resource.                               #
#                                                                   #
#####################################################################
sub create_source{

    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.source`;
    if ($? != 0){
        print "Error -- copying the source resource to $LIMTAR failed, exiting.\n";
        return $NOK;
    } else {
        print "Successfully copied the source resource $attrs{'name'}, continuing.\n";
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   create_ramdisk                                      #
#                                                                   #
# Description:  Copy ramdisk resource.                              #
#                                                                   #
#####################################################################
sub create_ramdisk{

    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.ramdisk`;
    if ($? != 0){
        print "Error -- copying the ramdisk resource to $LIMTAR failed, exiting.\n";
        return $NOK;
    } else {
        print "Successfully copied the ramdisk resource $attrs{'name'}, continuing.\n";
    }
     return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   create_map                                          #
#                                                                   #
# Description:  Copy map resource.                                  #
#                                                                   #
#####################################################################
sub create_map {

    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.map`;
    if ($? != 0){
        print "Error -- copying the map resource to $LIMTAR failed, exiting.\n";
        return $NOK;
    } else {
        print "Successfully copied the map resource $attrs{'name'}, continuing.\n";
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   create_exit                                         #
#                                                                   #
# Description:  Copy exit resource.                                 #
#                                                                   #
#####################################################################
sub create_exit{

    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.exit`;
    if ($? != 0){
        print "Error -- copying the exit resource to $LIMTAR failed, exiting.\n";
        return $NOK;
    } else {
        print "Successfully copied the exit resource $attrs{'name'}, continuing.\n";
    }
    return $OK;
}

#####################################################################
#                                 
# Subroutine:   create_postinstall
#                                 
# Description:  Allocate a post install script.
#                                 
#####################################################################
sub create_postinstall{

    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.postinstall`;
    if ($? != 0){
        print "Error -- allocated the postinstall resource to $LIMTAR failed, exiting.\n";
        return $NOK;
    } else {
        print "Successfully allocated the postinstall resource $attrs{'name'}, continuing.\n";
    }
    return $OK;
}


#####################################################################
#                                                                   #
# Subroutine:    create_rpm                                         #
#                                                                   #
# Description:   Copy an rpm resource.                              #
#                                                                   #
#####################################################################
sub create_rpm {

    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.rpm`;
    if ($? != 0){
        print "Error -- copying the rpm resource to $LIMTAR failed, exiting.\n";
        return $NOK;
    }

    @line=&read_rpm_table("$attrs{'source_path_name'}");
    foreach $line(@line){
        chop($line);
        $rpmfn = $line;
        `$LS $LIMRPM/$rpmfn*.rpm > /dev/null 2>&1`;
        if ($? != 0){
            print "Error -- the file $LIMRPM/$rpmfn does not exist, exiting.\n";	
            `$RM $LIMTAR/$attrs{'name'}.rpm`;
            return $NOK;
        }
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:    create_disk                                        #
#                                                                   #
# Description:    Copy a disk partition table.                      #
#                                                                   #
#####################################################################
sub create_disk {
  
    print "About to create the disk partition table resource.\n";
    `$CP $attrs{'source_path_name'} $LIMTAR/$attrs{'name'}.disk`;
    if ($? != 0){
        print "Error -- copying the disk partition table to $LIMTAR/tar failed, exiting.\n";
        return $NOK;
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:     write_resource_obj                                #
#                                                                   #
# Description:    Write a resource object.                          #
#                                                                   #
#####################################################################
sub write_resource_obj {

    if ($attrs{'type'} eq $RESOURCE_TYPE{source}){
	if (! defined $attrs{'target_location'}) {
		# If the target wasn't given, use the source as the target.
		$attrs{'target_location'}=$attrs{'source_path_name'};
	}
        `$ECHO "$attrs{'source_path_name'}:$attrs{'target_location'}" > $LIMDIR/$attrs{'name'}.$attrs{'type'}`;
    } else {
        `$ECHO "$attrs{'source_path_name'}" > $LIMDIR/$attrs{'name'}.$attrs{'type'}`;
    } 

    if ($? != 0){
        return $NOK;
    } else {
        print "Successfully created the $attrs{'type'} resource named $attrs{'name'}.\n";
    }
    return $OK;
}
#################### End lui_make_resource subroutines ########################

#####################################################################
#                                                                   #
# Subroutine:   lui_allocate_resource                               #
#                                                                   #
# Description:  Allocate an installation resource.                  #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#                                                                   #
#####################################################################
sub lui_allocate_resource {
    local (*flags) = @_;
    local ($rsrc,@rsrclist,$mach,@machlist);
 
    # Resolve the resource groups if given
    if ($flags{r}) {
        @rsrclist=&luiapi::lui_resolve_group("resource",$flags{r});
        if( ! defined $rsrclist[0] ) {
            print "Error -- Unable to resolve resource groups!\n";
            return $NOK;
        }
    } else {
        @rsrclist=split(/,/,$flags{n});
    }

    # Resolve the machine groups if given
    if ($flags{g}) {
        @machlist=&luiapi::lui_resolve_group("machine",$flags{g});
        if( ! defined $machlist[0] ) {
            print "Error -- Unable to resolve machine groups!\n";
            return $NOK;
        }
    } else {
        @machlist=split(/,/,$flags{m});
    }

    # Now process the input
    foreach $mach (@machlist){
	$mach=~s/^\s+//; $mach=~s/\s+$//;
        $flags{m}=$mach;
        # see if the machine exists
        if (-f "$LIMDIR/$flags{m}.client"){
        } else {
            print "Error -- a client machine named $flags{m} does not exist, exiting.\n";
            return $NOK;
        }

        if (&check_node_state){
        	return $NOK;
        }
        foreach $rsrc (@rsrclist){
            $rsrc=~s/^\s+//; $rsrc=~s/\s+$//;
            $flags{n}=$rsrc;

            # see if the resource exists
            if ( !($flags{t}=&resource_exists($flags{n})) ) {
                print "Error -- a resource named $flags{n} does not exist, exiting\n";
                return $NOK;
            }

            if (&write_client_allocate_table){
            	return $NOK;
            }

            print "Resource $flags{n} of type $flags{t} successfully allocated to client $flags{m}.\n";
        }
    }

    return $OK;

} # End lui_allocate_resource

#################### Start lui_allocate_resource subroutines ################

#####################################################################
#                                                                   #
# Subroutine:    check_node_state                                   #
#                                                                   #
# Description:    Make sure the state of the node is 'install'.     #
#                                                                   #
#####################################################################
sub check_node_state {

    # populate the machineObject hash
    &lui_read_machine_object ($flags{m},$CLIENT);

    # assign variable names to machineObject values ( why ??)
    &assign_client_values($flags{m});

    # Note: the client machine 'type' attribute could be either
    # MAINT_MODE, DISK_MODE, or INSTALL_MODE

    #
    # if the node is not set to install, return with error
    #
    if ($type ne $INSTALL_MODE){
        print "Error -- the client node $flags{m} is not set to install, exiting.\n";
        return $NOK;
    }
    return $OK;
}  # End check_node_state

#####################################################################
#                                                                   #
# Subroutine:     write_client_allocate_table                       #
#                                                                   #
# Description:    Write a client allocation table.                  #
#                                                                   #
#####################################################################
sub write_client_allocate_table {
  
    # Check if its already allocated
    `$GREP "^$flags{n}.$flags{t}:$flags{t}" /$LIMDIR/$flags{m}.allocate_table >/dev/null 2>&1`;
    if ( $? == 0 ) {
	return $OK;
    }
    `$ECHO $flags{n}.$flags{t}:$flags{t} >> /$LIMDIR/$flags{m}.allocate_table`;
    if ($? == 0){
        return $OK;
    } else {
        print "Error -- could not write to allocation table for $flags{m}, exiting.\n";
        return $NOK;
    }
    return $OK;
}  # End write_client_allocate_table

#################### End lui_allocate_resource subroutines ################

#####################################################################
#                                                                   #
# Subroutine:   lui_unallocate_resource                             #
#                                                                   #
# Description:  Unallocate an installation resource.                #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#                                                                   #
#####################################################################
sub lui_unallocate_resource {
    local (*flags) = @_;
    local ($rsrc,@rsrclist,$mach,@machlist);
    local $fail=0;
 
    # Resolve the resource groups if given
    if ($flags{r}) {
        @rsrclist=&luiapi::lui_resolve_group("resource",$flags{r});
        if( ! defined $rsrclist[0] ) {
            print "Error -- Unable to resolve resource groups!\n";
            return $NOK;
        }
    } else {
        @rsrclist=split(/,/,$flags{n});
    }

    # Resolve the machine groups if given
    if ($flags{g}) {
        @machlist=&luiapi::lui_resolve_group("machine",$flags{g});
        if( ! defined $machlist[0] ) {
            print "Error -- Unable to resolve machine groups!\n";
            return $NOK;
        }
    } else {
        @machlist=split(/,/,$flags{m});
    }

    # Now process the input
    foreach $mach (@machlist) {
        $mach=~s/^\s+//; $mach=~s/\s+$//;
        $flags{m}=$mach;
        if (-f "$LIMDIR/$flags{m}.client"){
        } else {
            print "Error -- a client machine named $flags{m} does not exist, exiting.\n";
            $fail=1;
            next;
        }

        # see if all resources should be removed
        if (&check_for_all){
            $fail=1;
            next;
        }

        foreach $rsrc (@rsrclist) { 
            $rsrc=~s/^\s+//; $rsrc=~s/\s+$//; 
            $flags{n}=$rsrc;
            # just delete specific resource
            if (&modify_client_allocate_table){
                $fail=1;
                next;
            }
	}
        print "Successfully unallocated resource(s) for client $flags{m}.\n";
    }
    if ($fail==0) {
        return $OK;
    }else{
        return $NOK;
    }

} # End lui_unallocate_resource

################  Start lui_unallocate_resource subroutines ###############

#####################################################################
#                                                                   #
# Subroutine:    check_for_all                                      #
#                                                                   #
# Description:    De-allocate all resources.                        #
#                                                                   #
#####################################################################
sub check_for_all {

    if ($flags{a}){
        `$RM $LIMDIR/$flags{m}.allocate_table 2>/dev/null`;
        if ($? != 0){
            print "Error -- either there are no resources to de-allocate from $flags{m}, or no resource allocation table for that machine, exiting.\n";
            return $NOK;
        } else {
            print "Successfully de-allocated all resources from $flags{m}.\n";
        }
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:     modify_client_allocate_table                      #
#                                                                   #
# Description:    Modify a client allocation table.                 #
#                                                                   #
#####################################################################
sub modify_client_allocate_table {

    if ($flags{t} eq $UNALL){
        return $NOK;
    }

    @line=&allocated_resources("$LIMDIR/$flags{m}.allocate_table");
    $allocated=0; 
    $deleted=0;
    foreach $line(@line){
        chop ($line);
        ($name, $type)= split (/:/,$line);
        ($realname,$rest)=split(/\./,$name);
        if ($realname eq $flags{n}){
            $deleted++;
            print "De-allocating the $flags{t} resource $realname from $flags{m}.\n";
        } else {
            $allocated++;
            `$GREP "$name:$type" /tmp/lim.$$ >/dev/null 2>&1`;
            if ( $? != 0 ) {
                `$ECHO "$name:$type" >>/tmp/lim.$$`;
            }
        }
    }

    if ($deleted == 0){
        print "The resource $flags{n} is not allocated to $flags{m}, exiting.\n";
        return $NOK;
    }

    if ($allocated > 0){
        `$CP /tmp/lim.$$ $LIMDIR/$flags{m}.allocate_table`;
        `$RM /tmp/lim.$$`;
    } else {
        `$RM $LIMDIR/$flags{m}.allocate_table`;
    }
    return $OK;
}

#################### End lui_unallocate_resource subroutines ###############

#####################################################################
#                                                                   #
# Subroutine:   lui_change_install_mode                             #
#                                                                   #
# Description:   Change the mode of a client machine to either:     #
#                   install, maint or disk.                         #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#                                                                   #
#####################################################################
sub lui_change_install_mode {
    local (*flags) = @_;
    local ($mach,@machlist);
    local $fail=0;
 
    # Resolve the machine groups if given
    if ($flags{g}) {
        @machlist=&luiapi::lui_resolve_group("machine",$flags{g});
        if( ! defined $machlist[0] ) {
            print "Error -- Unable to resolve machine groups!\n";
            return $NOK;
        }
    } else {
        @machlist=split(/,/,$flags{n});
    }

    foreach $mach (@machlist) {
        $mach=~s/^\s+//; $mach=~s/\s+$//;
        $flags{n}=$mach;
        if (!-f "$LIMDIR/$flags{n}.client"){
            print "Error -- the machine object $flags{n} does not exist or is not a client object, exiting.\n";
            $fail=1;
            next;
        }

        # populate the machineObject hash
        &lui_read_machine_object ($flags{n},$CLIENT);

        # assign variable names to machineObject values ( why ??)
        &assign_client_values($flags{n});

        # Note: the client machine 'type' attribute could be either
        # MAINT_MODE, DISK_MODE, or INSTALL_MODE

        if ($type eq $flags{m}){
            print "Error -- cannot set $flags{n} to $flags{m} mode, it is already set to $flags{m}, exiting.\n";
            $fail=1;
            next;
        }

        if ($flags{m} eq $INSTALL_MODE){
            if (&set_install){
                $fail=1;
                next;
            }
        } elsif ($flags{m} eq $MAINT_MODE){

            if (&set_maint){
                $fail=1;
                next;
            }
        } else {
            if (&set_disk) {
                $fail=1;
                next;
            }
        }

        print "Successfully changed $flags{n} to $flags{m} mode.\n";
    }
    if ($fail==0) {
        return $OK;
    }else{
        return $NOK;
    }

} # End lui_change_install_mode

##########  Start lui_change_install_mode subroutines  ###################

#####################################################################
#                                                                   #
# Subroutine:     set_install                                       #
#                                                                   #
# Description:    Change a client machine to the 'install' mode.    #
#                                                                   #
#####################################################################
sub set_install(){

    # Note: the client attr values were set by the caller
    if(&delete_bootp_entry ($ha,$flags{n})) {
        return $NOK;
    }

    if (&add_bootp_entry($ip,$flags{n},$gateway,$hostname,$netmask,$ha)){
        return $NOK;
    }

    `$RM /$LIMDIR/$flags{n}.client`;

    $machine_line="$ip:$netmask:$ha:$INSTALL_MODE:$adapter:$default:$hostname:$gateway:$num_proc:$pbsstring:$mac_id:$reboot:";

    &write_client_obj("$LIMDIR/$flags{n}.client",$machine_line);
    if ($? == 0){
        return $OK;
    } else {
        return $NOK;
    }
    return $OK;
}

#####################################################################
#                                                                   #
# Subroutine:   set_maint                                           #
#                                                                   #
# Description:  Change a client machine to the 'maint' mode.        #
#                                                                   #
#####################################################################
sub set_maint(){

    if (-f "$LIMDIR/$flags{n}.allocate_table" && !$flags{f}){
        print "Error -- you cannot change the mode of a client with allocated resources to $MAINT_MODE, you must either use unallimr to de-allocate the resources first, or run chlimmod with the -f (force) option, exiting.\n";
        return $NOK;
    }

    if(&delete_bootp_entry ($ha,$flags{n})){
        return $NOK;
    }

    if(&add_bootp_entry($ip,$flags{n},$gateway,$hostname,$netmask,$ha)){
        return $NOK;
    }

    `$RM /$LIMDIR/$flags{n}.client`;

    $machine_line="$ip:$netmask:$ha:$MAINT_MODE:$adapter:$default:$hostname:$gateway:$num_proc:$pbsstring:$mac_id:$reboot:";

    &write_client_obj("$LIMDIR/$flags{n}.client",$machine_line);
    if ($? == 0){
        return $OK;
    } else {
        return $NOK;
    }
  
    return 0;
}

#####################################################################
#                                                                   #
# Subroutine:   set_disk                                            #
#                                                                   #
# Description:  Change a client machine to the 'maint' mode.        #
#                                                                   #
#####################################################################
sub set_disk(){

    if (-f "$LIMDIR/$flags{n}.allocate_table" && !$flags{f}){
        print "Error -- you cannot change the mode of a client with allocated resources to $DISK_MODE, you must either use unallimr to de-allocate the resources first or run chlimmod with the -f (force) option, exiting.\n";
        return $NOK;
    }

    &delete_bootp_entry ($ha,$flags{n});
    if ($? != 0){
        return $NOK; 
    }

    `$RM /$LIMDIR/$flags{n}.client`;

    $machine_line="$ip:$netmask:$ha:$DISK_MODE:$adapter:$default:$hostname:$gateway:$num_proc:$pbsstring:$mac_id:$reboot:";

    &write_client_obj("$LIMDIR/$flags{n}.client",$machine_line);
    if ($? == 0){
        return $OK;
    } else {
        return $NOK;
    }
}

##########  End lui_change_install_mode subroutines  ###################

#####################################################################
#                                                                   #
# Subroutine:   lui_list_machine                                    #
#                                                                   #
# Description:  List a client or server machine information.        #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_list_machine {
    local (*flags) = @_;
    local ($mach,@machlist,$type);
    local $fail=0;
 
    # Resolve the machine groups if given
    if ($flags{g}) {
        @machlist=&luiapi::lui_resolve_group("machine",$flags{g});
        if( ! defined $machlist[0] ) {
            print "Error -- Unable to resolve machine groups!\n";
            return $NOK;
        }
    } else {
        @machlist=split(/,/,$flags{n});
    }
    # If no machines were listed, show all machines.
    if (!defined $machlist[0]) {
       foreach $type ($SERVER,$CLIENT){
          $hdg="$type machines:";
          foreach $mach (`$LS $LIMDIR/*.$type 2>/dev/null` ) {
               # This is so we only print the heading for populated types
               if (defined $hdg){
                   printf "$hdg\n";
                   undef $hdg;
               }
               chop $mach;
               $mach=~s/^$LIMDIR\///;
               $mach=~s/\.$type$//;
               printf "\t$mach\n";
          }   
       }
    }

    foreach $mach (@machlist) {
        $mach=~s/^\s+//; $mach=~s/\s+$//;
        $flags{n}=$mach;

        if ( ! ($flags{t}=&machine_exists($flags{n})) ) {
            print "error -- The machine '$flags{n}' does not exist, exiting\n";
            $fail=1;
            next;
        }

       if ($flags{t} eq $CLIENT){
            if (&display_machine_info){
            $fail=1;
            next;
            }
            if (&display_resource_info){
            $fail=1;
            next;
            }
            if (&display_bootptab_entry){
            $fail=1;
            next;
            }
        }

        if ($flags{t} eq $SERVER){
            if (&display_server_info){
            $fail=1;
            next;
            }
            if (&display_server_clients){
            $fail=1;
            next;
            }
            if (&display_server_resources){
            $fail=1;
            next;
            }
        }
    }

    if ($fail==0) {
        return $OK;
    }else{
        return $NOK;
    }
}

#####################################################################
#   Start lui_list_machine   subroutines                            # 
#------------------------------------------------------------------ #
# Subroutines:   display_machine_info display_resource_info         #
#                display_bootptab_entry display_server_info         #
#                display_server_clients display_server_resources    #
#####################################################################

sub display_machine_info{            # Display machine object info
  luiapi::lui_read_machine_object($flags{n},$CLIENT);
  &assign_client_values($flags{n});
  print "\nclient name     =       $flags{n}\n";
  print "       ip address      =       $ip\n";
  print "       netmask         =       $netmask\n";
  print "       MAC address     =       $ha\n";
  print "       MAC id          =       $mac_id\n";
  print "       client mode     =       $type\n";
  print "       install adapter =       $adapter\n";
  print "       default route   =       $default\n";
  print "       long hostname   =       $hostname\n";
  print "       install gateway =       $gateway\n";
  print "       # of processors =       $num_proc\n";
  print "       PBS string      =       $pbsstring\n";
  print "       auto reboot     =       $reboot\n";


  return $OK;
}

sub display_resource_info{         # Display machine resources table
  if (-f "$LIMDIR/$flags{n}.allocate_table"){
        print "\nthe resources allocated to $flags{n} are:\n";
        print "         resource name           type\n";
        print "         -------------           ----\n";
        @line=&allocated_resources("$LIMDIR/$flags{n}.allocate_table");
        foreach $line(sort bytype @line){
                chop($line);
                ($name, $type)= split (/:/,$line);
                ($realname,$rest)=split(/\./,$name);
		printf "         %-23.23s %s\n",$realname,$type;

        }
  }
  else {
        print "\n$flags{n} has no resources allocated\n";
  }
  return $OK;
}

sub display_bootptab_entry{       # Display booth table entry
  $line=`$GREP $ha /etc/bootptab`;
  if ($line eq ""){
        print "\n$flags{n} has no entry in the /etc/bootptab file\n";
  }
  else {
        print "\nthe entry for $flags{n} in /etc/bootptab is:\n$line\n";
  }

  return $OK;
}

sub display_server_info{

  luiapi::lui_read_machine_object($flags{n},$SERVER);

 &assign_server_values($flags{n});
  print "\nserver name = $flags{n}\n";
  print "       ip address      =       $server_ipaddr\n";
  print "       netmask         =       $server_netmask\n";
  print "       # of processors =       $server_numproc\n";
  print "       PBS string      =       $server_pbsstring\n";
  print "       client auto-\n";
  print "       reboot default  =       $server_reboot\n";

  return $OK;
}

sub display_server_clients{
  @line=`cd $LIMDIR; $LS *.client 2>/dev/null`;
  if (chop(@line) eq  ""){
        print "\nthere are no clients for server $flags{n}\n";
  }
  else{
        print "\nthe list of all clients for $flags{n} is:\n";
        foreach $line(@line){
                ($name,$rest)=split(/\./,$line);
                print "         $name\n";
        }
  }
  return $OK;
}

sub display_server_resources{
  @line=`cd $LIMDIR; $LS *.rpm *.exit *.kernel *.map *.file *.disk *.source  2>/dev/null`;
  if (chop(@line) eq  ""){
        print "\nthere are no resources for server $flags{n}\n";
  }
  else{
        print "\nthe list of all resources for $flags{n} is:\n";
        print "         resource name           type\n";
        print "         -------------           ----\n";
        foreach $line(sort bytypesrv @line){
                ($name,$type)=split(/\./,$line);
		printf "         %-23.23s %s\n",$name,$type;
        }
  }
  return $OK;
}

############ End lui_list_machine subroutines ###################

#####################################################################
#                                                                   #
# Subroutine:   lui_list_resource                                   #
#                                                                   #
# Description:  List a client or server machine recources.          #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_list_resource {
   local (*flags) = @_;
    local ($rsrc,@rsrclist,$type,$hdg);
   local $fail=0;
 
    # Resolve the resource groups if given
    if ($flags{r}) {
        @rsrclist=&luiapi::lui_resolve_group("resource",$flags{r});
        if( ! defined $rsrclist[0] ) {
            print "Error -- Unable to resolve resource groups!\n";
            return $NOK;
        }
    } else {
        @rsrclist=split(/,/,$flags{n});
    }

    # If no resources were listed, show all resources.
    if (!defined $rsrclist[0]) {
       foreach $type (sort(values %RESOURCE_TYPE)){
          $hdg="$type resources:";
          foreach $rsrc (`$LS $LIMDIR/*.$type 2>/dev/null` ) {
               # This is so we only print the heading for populated types
               if (defined $hdg){
                   printf "$hdg\n";
                   undef $hdg;
               }
               chop $rsrc;
               $rsrc=~s/^$LIMDIR\///;
               $rsrc=~s/\.$type$//;
               printf "\t$rsrc\n";
          }   
       }
    }

    foreach $rsrc (@rsrclist){
        $rsrc=~s/^\s+//; $rsrc=~s/\s+$//;
        $flags{n}=$rsrc;
        # Is this a valid resource type?
        if ($flags{t} && (! grep(/$flags{t}/,values %RESOURCE_TYPE) ) ){
            print "error -- You must specify a valid resource type with the -t option.\n";
            &print_usage();
            $fail=1;
            next;
        }

        # see if the resource exists
        if ( !($flags{t}=&resource_exists($flags{n})) ) {
            print "Error -- a resource named $flags{n} does not exist, exiting\n";
            $fail=1;
            next;
        }

        if (&display_general_info){
            $fail=1;
            next;
        }
    }

    if ($fail==0) {
        return $OK;
    }else{
        return $NOK;
    }
}

#####################################################################
#   Start lui_list_resource  subroutines                            # 
#------------------------------------------------------------------ #
# Subroutines:   display_general_info                               #
#                                                                   #
#####################################################################

sub display_general_info {
 # local $line;
  $line=&read_resource_object("$LIMDIR/$flags{n}.$flags{t}");
  chop($line);
  if ($flags{t} eq $RESOURCE_TYPE{source}){
        ($line, $junk)= split(/:/,$line);
  }
  print "\nthe resource $flags{n} of type $flags{t} is associated with file:\n";
  print "       $line\n";

  if (-r "$LIMTAR/$flags{n}.$flags{t}") {
    `$DIFF $line $LIMTAR/$flags{n}.$flags{t}`;
    if ($? != 0){
        print "\nthe file is different than the resource stored in the LIM database; you may want to change the resource\n";
    }
    else {
        print "\nthe file and the resource as stored in the LIM database are identical\n";
    }
  }

  &am_i_allocated($flags{n}, $flags{t});
  return $OK;
 
} #-------------- End display general info

########### End lui_list_resource subroutine ######################

#####################################################################
#                                                                   #
# Subroutine:   lui_delete_machine                                  #
#                                                                   #
# Description:  Deletes a machine                                   #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_delete_machine{
   local (*flags) = @_;
    local ($mach,@machlist);
    local $fail=0;
 
    # Resolve the machine groups if given
    if ($flags{g}) {
        @machlist=&luiapi::lui_resolve_group("machine",$flags{g});
        if( ! defined $machlist[0] ) {
            print "Error -- Unable to resolve machine groups!\n";
            return $NOK;
        }
    } else {
        @machlist=split(/,/,$flags{n});
    }

    foreach $mach (@machlist) {
        $mach=~s/^\s+//; $mach=~s/\s+$//;
        $flags{n}=$mach;

        if ( ! ($flags{t}=&machine_exists($flags{n})) ) {
            print "error -- The machine '$flags{n}' does not exist, exiting\n";
            $fail=1;
            next;
        }


        if ($flags{t} eq $CLIENT){
            if (&check_if_allocated){     # all resources must be unallocated
                $fail=1;
                next;
            }

            if (&delete_client_files($flags{n})){
                $fail=1;
            }
            if (&delete_bootp_entry($ha, $flags{n})){
                $fail=1;
            }
            if (&delete_machine_obj($flags{n},$flags{t})){
                $fail=1;
            }
        }

        if ($flags{t} eq $SERVER){
            # First delete all of the client info.
            @client =`cd $LIMDIR;$LS *.client 2>/dev/null`;
            foreach $client(@client){
                    chop($client);
                    ($name,$type)=split(/\./,$client);
                    &delete_client_files($name);
                    &delete_bootp_entry($ha,$name);
            }
            if (&delete_server_files){
                $fail=1;
                next;
            }
        }
    }
    if ($fail==0) {
        return $OK;
    }else{
        return $NOK;
    }
}
#####################################################################
#   Start lui_delete_machine subroutines                            # 
#------------------------------------------------------------------ #
# Subroutines:   check_if_allocated delete_client_files             #
#                delete_bootp_entry delete_machine_obj              #
#####################################################################

sub check_if_allocated{
#  local ($name)=@_;
#  if (-f "$LIMDIR/$name.allocate_table"){
  if (-f "$LIMDIR/$flags{n}.allocate_table"){
#        print "error -- the machine $name of type $option{t} has resources allocated to it and cannot be deleted until the resources are de-allocated, use unallimr\n";
        print "error -- the machine $flags{n} of type $flags{t} has resources allocated to it and cannot be deleted until the resources are de-allocated, use unallimr\n";
        return $NOK;
  }
  return $OK;
}  #---------------- End check_if_allocacted

sub delete_client_files{
  luiapi::lui_read_machine_object($flags{n},$CLIENT);
  &assign_client_values($flags{n});
  if ($ip eq ""){
        print "error -- something is wrong with the client file for $flags{n}, exiting rather than destroying your filesystems\n";
        return $NOK;
  }
  print "about to delete the remote root file system for $flags{n}\n";
  `$RM -R /tftpboot/$ip/*`;
  if ($? != 0) {
        print "error -- could not remove remote root filesystem at /tftpboot/$ip, exiting\n";
        return $NOK;
  }
  `$RMDIR /tftpboot/$ip`;
  return $OK;
}   #----------------- End delete_client_flies

sub delete_server_files{
  `$RM -R /tftpboot/lim/*`;             # I hardcode the path names here for
  `$RMDIR /tftpboot/lim`;
   print "successfully deleted the machine object for server $option{n}\n";
  `$RM -R /tftpboot/tar/* 2>/dev/null`;         # safety
  `$RMDIR /tftpboot/tar`;
  `$RM -R /tftpboot/common/* 2>/dev/null`;
  `$RMDIR /tftpboot/common`;
   return $OK;
}  #------------------ End delete_server_files

sub delete_machine_obj{
  `$RM /tftpboot/lim/$flags{n}.$flags{t}`;
  if ($? != 0){
     print "error -- could not delete the machine object for $flags{t} $flags{n}, exiting\n";
     return $NOK;
  }
  print "successfully deleted the machine object for $flags{t} $flags{n}\n";
  return $OK;
}   #------------------ End delete_machine_obj

############# End lui_delete_machine subroutine ####################


#####################################################################
#                                                                   #
# Subroutine:   lui_delete_resource                                 #
#                                                                   #
# Description:  Deletes a machine's resources                       #
#                                                                   #
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_delete_resource{
   local (*flags) = @_;
    local ($rsrc,@rsrclist);
   local $fail=0;
 
    # Resolve the resource groups if given
    if ($flags{r}) {
        @rsrclist=&luiapi::lui_resolve_group("resource",$flags{r});
        if( ! defined $rsrclist[0] ) {
            print "Error -- Unable to resolve resource groups!\n";
            return $NOK;
        }
    } else {
        @rsrclist=split(/,/,$flags{n});
    }

    foreach $rsrc (@rsrclist){
        $rsrc=~s/^\s+//; $rsrc=~s/\s+$//;
        $flags{n}=$rsrc;

        # see if the resource exists
        if ( !($flags{t}=&resource_exists($flags{n})) ) {
            print "Error -- a resource named $flags{n} does not exist, exiting\n";
            $fail=1;
            next;
        }
   
        if (&check_allocated_resource){    
            $fail=1;
            next;
        }

        if (&delete_resource_obj($flags{n},$flags{t})){
            $fail=1;
            next;
        }
    }
    if ($fail==0) {
        return $OK;
    }else{
        return $NOK;
    }
}

#####################################################################
#   Start lui_deleteresources subroutines                           # 
#------------------------------------------------------------------ #
# Subroutines:   check_allocated_resource delete_resource_obj       #
#                                                                   #
#####################################################################
sub check_allocated_resource {
  @array=<$LIMDIR/*.allocate_table>;
  foreach $a(@array){
        @line=&allocated_resources ("$a");
        foreach $line (@line){
                chop($line);
                ($name,$rest)=split(/\./,$line);
                ($rest,$type)=split(/:/,$line);
                if ($flags{n} eq $name && $flags{t} eq $type){
                    print "error -- you cannot delete the resource $flags{n} of type $flags{t} because it is allocated in $a, exiting\n";
                    return NOK;
                }
        }
  }
  return $OK;
}   #----------------- End check_allocated_resource

sub delete_resource_obj{

    my $fileext;
    my $resourcetype;
    unlink "$LIMDIR/$flags{n}.$flags{t}";
    if ($? == 0){
        print "the resource $flags{n} of type $flags{t} was successfully deleted\n";
    }
    else{
          print "error -- could not delete the resource $flags{n} of type $flags{t}, exiting\n";
        return $NOK;
    }
    if ($flags{t} eq "file") {
	$fileext = "tgz";
	$resourcetype = "tar";
    }
    else{
        $fileext = $resourcetype = $flags{t};
    } 
    if (-e "$LIMTAR/$flags{n}.$fileext"){
        unlink "$LIMTAR/$flags{n}.$fileext";
         if ($? == 0){
            print "the $filetype file $LIMTAR/$flags{n}.$fileext was successfully deleted\n";
         }
         else{
             print "error -- could not delete the file $LIMTAR/$flags{n}.$fileext\n";
         }
    }
    else{
        print "there is no $LIMTAR/$flags{n}.$fileext file associated with this resource, so none will be removed\n";
    }
    return $OK;
}   #----------------- End delete_resource_obj

############## End lui_delete_resource subroutine ##################

#####################################################################
#                                                                   #
# Subroutine:   lui_make_group
#                                                                   #
# Description:  Create/add to machine & resource groups
#                                                                   #
# Input:        The %flags hash, the following keys are referenced
#			m	List of members to work with
#			g 	List of groups to work with
#			t	Type of groups (machine,resource)
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_make_group {
	local (*flags) = @_;
	local @targetlist;
	local @grouplist;
	my $target;
	my $group;
	my @newgrp;
	my $failure=0;
	my $gtype;

	# Read in the current group definitions
	&read_groups;

	if ($flags{m}) {
		# Parse the target list
		@targetlist=split(',',$flags{m});
	} else {
		printf "error -- You must specify a member list.\n";
		$failure=1;
	}

	if ($flags{g}) {
		# Parse the group list
		@grouplist=split(',',$flags{g});
	} else {
		printf "error -- You must specify a group list.\n";
		$failure=1;
	}

	if ($flags{t} ne "machine" &&  $flags{t} ne "resource") {
		printf "error -- You must specify a valid group type.\n";
		$failure=1;
	}

	# Check the types of the groups
	foreach $group (@grouplist){
		$gtype=&group_exists($group);
		if ( $gtype && $gtype ne $flags{t} ) {
			printf "error -- Group $group does not match the specified group type.\n";
			$failure=1;
		}
		if ($flags{t} eq "resource" ) {
			$group=$RGprefix.$group;
		}
		push(@newgrp,$group);
	}	
	@grouplist=@newgrp;

	# Check that the targets exist
	foreach $target (@targetlist)	{
		if ($flags{t} eq "resource") {
			if (! &resource_exists($target) ) {
				printf "Resource $target does not exist.\n";
				$failure=1;
			}
		}else {
			if (! &machine_exists($target) ) {
				printf "Machine $target does not exist.\n";
				$failure=1;
			}	
		}
	}
	if ($failure) {
		return $NOK;
	}
	# Add a list of targets to a group
	foreach $group (@grouplist) {
		foreach $target (@targetlist) {
			$Groups{$group}{$target}=1;
		}
	}
	# Write the groups to the file
	&write_groups;
	printf "Group(s) successfully added\n";
	return $OK;

} #lui_make_group


#####################################################################
#                                                                   #
# Subroutine:   lui_list_group
#                                                                   #
# Description:  List machine & resource groups
#                                                                   #
# Input:        The %flags hash, the following keys are referenced
#			g 	List of groups to work with
#				The "g" flag is optional, if given,
#				print all groups specified, otherwise
#				print all groups.
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_list_group {
	local (*flags) = @_;
	local @grouplist;
	my $group;
	my @newgrp;
	my $failure=0;
	my $gtype;

	# Read in the current group definitions
	&read_groups;

	if ($flags{g}) {
		# Parse the group list
		@grouplist=split(',',$flags{g});
	}

	# Check the types of the groups
	foreach $group (@grouplist){
		$gtype=&group_exists($group);
		if (! $gtype ) {
			printf "error -- Group $group does not exist.\n";
			$failure=1;
		} 
		if ($gtype eq "resource" ) {
			$group=$RGprefix.$group;
		}
		push(@newgrp,$group);
	}	
	@grouplist=@newgrp;

	if ($failure) {
		return $NOK;
	}
	# print the contents of the group specified
	# or a list of all groups.
	if (! defined @grouplist) {
		# Print a list of all groups
		&print_group();
	} else {
		# Print the members of the specified groups
		foreach $group (@grouplist) {
			&print_group($group);
		}
	}
	return $OK;

} #lui_list_group

#####################################################################
#                                                                   #
# Subroutine:   lui_delete_group
#                                                                   #
# Description:  Delete/remove from machine & resource groups
#                                                                   #
# Input:        The %flags hash, the following keys are referenced
#			g 	List of groups to work with
#			m	List of members to work with
#				The "m" flag is optional. If given,
#				delete the members specified from 
#				the groups, otherwise, delete the 
#				entire group.
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_delete_group {
	local (*flags) = @_;
	local @targetlist;
	local @grouplist;
	my $target;
	my $group;
	my @newgrp;
	my $failure=0;
	my $gtype;

	# Read in the current group definitions
	&read_groups;

	if ($flags{m}) {
		# Parse the target list
		@targetlist=split(',',$flags{m});
	} 

	if ($flags{g}) {
		# Parse the group list
		@grouplist=split(',',$flags{g});
	} else {
		printf "error -- You must specify a group list.\n";
		$failure=1;
	}

	# Check the types of the groups
	foreach $group (@grouplist){
		$gtype=&group_exists($group);
		if ( ! $gtype ) {
			printf "error -- Group $group does not exist.\n";
			$failure=1;
		}
		if ($gtype eq "resource" ) {
			$group=$RGprefix.$group;
		}
		push(@newgrp,$group);
	}	
	@grouplist=@newgrp;

	if ($failure) {
		return $NOK;
	}

	# Delete a group
	foreach $group (@grouplist) {
		# If a target list was specified, just remove the
		# targets from the group. Otherwise, delete the whole
		# group.
		if ($flags{m}) {
			foreach $target (@targetlist) {
				undef $Groups{$group}{$target};
			}
		} else {
			undef $Groups{$group};
		}
		# Write the groups to the file
		&write_groups;
	}
	printf "Group(s) successfully deleted\n";
	return $OK;

} #lui_delete_group

sub lui_resolve_group {
	# Resolve group subroutine: Resolve a list of 
	# LUI groups to their component members
	# Usage : &resolve_group(<type>,<grouplist>)
	#	<type> Group type, "resource" or "machine"
	#	<grouplist> An array(@groups) of the group names
	#
	# Returns: A list (@members) of the members of all the groups
	#	in <grouplist>.

	my $gtype=shift(@_);
	my @groups=split(/,/,$_[0]);
	my $group,$member,@members;
	undef @members;
	#read the groups from the file
	&read_groups;
	# Go through each entry
	foreach $group (@groups) {
		# Add the resource prefix to resource groups
		if ($gtype eq "resource") {
			$group=$RGprefix.$group;
		}
		# If this entry is a member of the requested group(s) save it.
		foreach $member (sort keys %{$Groups{$group}}) {
			push(@members,$member);
		}
	}
	# Return all the matches
	return &unique(@members);
}

sub read_groups {
	# Read the groups in from the file. Store it in the global
	# hash %Groups. 
	my $group,$member;
	undef %Groups;
	# Open the file and read the groups into the hash
	open (TABLE,"$Groupfile");
	while (<TABLE>) {
		chop;
		($group,$member)=split(',',$_);
		$Groups{$group}{$member}=1;
	}
	close TABLE;
} #read_groups

sub write_groups {
	# Write the groups from the hash out to the file.
	my $group,$member;
	# Open the temp file and write out the new groups
	open (TABLE,">$Groupfile.tmp");
	# Go through the hash and write out each entry
	foreach $group (sort keys %Groups) {
		foreach $member (sort keys %{$Groups{$group}}) {
			if ($Groups{$group}{$member} == 1) {
				printf TABLE "$group,$member\n";
			}
		}
	}
	close TABLE;
	# Move the temp file to the actual file
	`/bin/mv $Groupfile.tmp $Groupfile`;
} #write_groups

                        
sub print_group {
	# Print the contents of a group or a list of groups
	my $mgroup=$_[0];
	my $group;
	my $member;
	my $first;
	my @Rgroups;
	if ( defined $mgroup ) {
		# If a group was specified, print its members
		$group=$mgroup;
		$group =~ s/^$RGprefix//;
		printf "Group: $group\n";
		foreach $member (sort keys %{$Groups{$mgroup}}) {
			printf "\t$member\n";
		}
	} else {
		# If no group was specified, print a list a groups
		$first=1;
		foreach $group (sort keys %Groups ) {
			if ($group =~ /^$RGprefix/) {
				push (@Rgroups,$group);
			} elsif ($Gtype ne "Resource") {	
				if ($first) {
					printf "Machine Groups:\n";
					$first=0;
				}
				printf "\t$group\n";
			}
		}
		$first=1;
		if ($Gtype ne "Machine") {
			foreach $group (@Rgroups) {
				$group =~ s/^$RGprefix//;
				if ($first) {
					printf "Resource Groups:\n";
					$first=0;
				}
				printf "\t$group\n";
			}
		}
	}
} #print_group
############## End lui_group subroutines ####################
#####################################################################
#                                                                   #
# Subroutine:   lui_make_clients
#                                                                   #
# Description:  Make a group of clients
#                                                                   #
# Input:        The %flags hash, the following keys are referenced
#			i	starting ip address
#			m	netmask
#			d(opt)	default route
#			g(opt)	install gateway
#			p(opt)	number of processors
#			s(opt)	PBS string
#			I(opt)	starting macid in /etc/MAC.info
#			c(opt)	number of clients to create
#			G(opt)	machine group to add clients to
#
#		The ip address for each client is determined by 
#		incrementing the previous ip address by 1.
#
#		The long host name is resolved from the ip addr.
#
#		The LUI object name is the 1st part of the hostname.
#
#		The sequence is determined by /etc/MAC.info. We will
#		start with the line that matches the value for -I and
#		continue for the value of -c lines.
#
#		If -I is not specified the default is the first macid.
#		If -c is not specified the default is the number of
#		entries to the end of /etc/MAC.info.
#
# Return Values:                                                    #
#                  NOK  - not successful  (1)                       #
#                   OK  - successful  (0)                           #
#                                                                   #
#####################################################################
sub lui_make_clients {
	local (*flags) = @_;
	local %attrs,%gopts,@MACIDS,$start_i,$end_i;
	local $i,$iaddr,$rest;
	local $fail=0;

	# The following are fixed
	$attrs{type}="client";
	$attrs{inst_adptr}="eth0";
	
	# These are just passed on to lui_make_machine
	$attrs{netmask}=$flags{m};
	$attrs{def_route}=$flags{d};
	$attrs{inst_gateway}=$flags{g};
	$attrs{num_proc}=$flags{p};
	$attrs{pbs_string}=$flags{s};
	$attrs{reboot}=$flags{r};

	if (&read_macinfo) {
		printf "Error - Unable to read $MACINFO file.\n";
		return $NOK;
	}

	undef $start_i;
	# If start id wasn't specified, use first id.
	if (! $flags{I} ) {
		$start_i=0;
	} else {
		# Otherwise, find the index of the start id.
		foreach $i (0..(scalar(@MACIDS)-1)) {
			if ($MACIDS[$i] eq $flags{I}) {
				$start_i=$i;
				last;
			}
		}
		if (! defined $start_i) {
			printf "Error - Starting MAC ID, $flags{I}, was not found in $MACINFO.\n";
			return $NOK;
		}
	}

	# If count wasn't specified, define all from start to end
	if (! $flags{c} ) {
		$end_i=scalar(@MACIDS)-1;
	} else {
		$end_i=$start_i+$flags{c}-1;
	}

	foreach $i ($start_i..$end_i) {
		$attrs{mac_id}=$MACIDS[$i];
		undef $attrs{mac_address};
		# Increment the ip addr unless this is the first client
		if ($i != $start_i) {
			$attrs{ip_address}=&inc_ip($attrs{ip_address})
		} else {
			$attrs{ip_address}=$flags{i};
		}
		# Get the hostname
		$iaddr = inet_aton($attrs{ip_address});
		$attrs{lhostname}=gethostbyaddr($iaddr, AF_INET);
		if ($attrs{lhostname} eq "") {
			print "Error, IP address $attrs{ip_address} is not resolvable!\n";
			$fail=1;
			next;
		}
			
		# Use the first part of the hostname for the client name
		($attrs{name},$rest)=split(/\./,$attrs{lhostname});
		
		if( &luiapi::lui_make_machine(\%flags, \%attrs) ) {
			print "Error creating machine object for $attrs{name}!\n";
			$fail=1;
			next;
		}
	}
	if ($fail) {
		return $NOK;
	} else {
		return $OK;
	}
	
} #lui_make_clients


# Read in the macinfo file to the @MACIDS array.
sub read_macinfo{
	my $line;
	my $MACid;
	my $MACaddr;
	if (! open (MACFILE, "<$MACINFO") ) {
		printf "Can't open $MACINFO: $!\n";
		return $NOK;
	}
	# Read all the lines in
	while ($line = <MACFILE>) {
		# If a line is blank or starts with #, skip it
		if ( ($line =~ /^\s*#/) || ($line =~ /^\s*$/) ) {
			next;
		}
		($MACid,$MACaddr)= split(/\s+/, $line);
		push(@MACIDS,$MACid)
	}
	return $OK;
} #read_macinfo

# Increment an IP address by 1, carrying to the previous octet as required.
sub inc_ip{
	my $ip=$_[0];
	local $a,$b,$c,$d;
	($a,$b,$c,$d)=split(/\./,$ip);
	$d++;
	if ($d > 255) {
		$d = $d - 256;
		$c++;
		if ($c > 255) {
			$c = $c - 256;
			$b++;
			if ($b > 255) {
				$b = $b - 256;
				$a++;
				if ($a > 255) {
					printf "Error - ip address out of valid range.\n";
					return $NOK;
				}
			}
		}
	}
	return $a."\.".$b."\.".$c."\.".$d;
} #inc_ip

# Set up the network configuration for a machine for a SuSE installation
# in /tftpboot/<ipaddr>/etc/rc.config.  This routine will use the rc.config
# that it had copied from the server's system unless there is a
# suse.rc.config in the lui directory, so that different defaults can be
# provided for the clients than those used on the server.

sub create_suseconfig{

# Copy the client's rc.config if different from the server's
  if (-e "$BASE/templates/suse.rc.config") {
    `$CP $BASE/templates/suse.rc.config /tftpboot/$attrs{'ip_address'}/etc/rc.config`;
    }

# Figure out the broadcast value
   ($network,$broadcast)=&network_ip($attrs{'ip_address'}, $attrs{'netmask'});

# Array of substitutions to make to the rc.config file.
# Note the the ? is used as a delimeter. Values in rc.config are enclosed
# in quotation marks.

  @configsubs=("s?^IPADDR_0.*?IPADDR_0=\\\"$attrs{ip_address}\\\"?",
               "s?^IPADDR_1.*?IPADDR_1=\\\"\\\"?",
               "s?^IPADDR_2.*?IPADDR_2=\\\"\\\"?",
               "s?^IPADDR_3.*?IPADDR_3=\\\"\\\"?",
               "s?^NETDEV_0.*?NETDEV_0=\\\"eth0\\\"?",
               "s?^NETDEV_1.*?NETDEV_1=\\\"\\\"?",
               "s?^NETDEV_2.*?NETDEV_2=\\\"\\\"?",
               "s?^NETDEV_3.*?NETDEV_3=\\\"\\\"?",
               "s?^IFCONFIG_0.*?IFCONFIG_0=\\\"\"$attrs{ip_address} broadcast $broadcast netmask $attrs{netmask} up\\\"\"?",
               "s?^IFCONFIG_1.*?IFCONFIG_1=\\\"\\\"?",
               "s?^IFCONFIG_2.*?IFCONFIG_2=\\\"\\\"?",
               "s?^IFCONFIG_3.*?IFCONFIG_3=\\\"\\\"?",
               "s?^FQHOSTNAME.*?FQHOSTNAME=\\\"$attrs{lhostname}\\\"?",
               "s?^FROM_HEADER.*?FROM_HEADER=\\\"$attrs{lhostname}\\\"?");

  $currentfile="/tmp/suseout";
  $nextfile="/tmp/suseout1";
  system("$CP /tftpboot/$attrs{'ip_address'}/etc/rc.config $currentfile");

# Make each substitution in the rc.config file, alternating between
# /tmp/suseout and /tmp/suseout1 output files.  When complete, move
# back to the /etc directory for the client.

  foreach $configsub(@configsubs) {
     system("$SED $configsub $currentfile > $nextfile");
     # Swap the currentfile and the nextfile.
     $savefile = $currentfile;
     $currentfile = $nextfile;
     $nextfile = $savefile;
     }

  system("$CP $currentfile /tftpboot/$attrs{'ip_address'}/etc/rc.config");
  `$RM /tmp/suseout`;
  `$RM /tmp/suseout1`;

  return 0;
} #create_suseconfig

############## End lui_make_clients subroutines ####################
############## End Luiapi.pm ################################
1;
